"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.models.components import dataset as components_dataset
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class Type(str, Enum):
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """

    EVALUATION = "evaluation"
    FINE_TUNING = "fine-tuning"


class GetDatasetsRequestTypedDict(TypedDict):
    project: str
    r"""Project Name associated with the datasets like `New Project`"""
    type: NotRequired[Type]
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """
    dataset_id: NotRequired[str]
    r"""Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`"""


class GetDatasetsRequest(BaseModel):
    project: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Project Name associated with the datasets like `New Project`"""

    type: Annotated[
        Optional[Type],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """

    dataset_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`"""


class GetDatasetsResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    testcases: NotRequired[List[components_dataset.DatasetTypedDict]]


class GetDatasetsResponseBody(BaseModel):
    r"""Successful response"""

    testcases: Optional[List[components_dataset.Dataset]] = None


class GetDatasetsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetDatasetsResponseBodyTypedDict]
    r"""Successful response"""


class GetDatasetsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetDatasetsResponseBody] = None
    r"""Successful response"""
